import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import java.io.File;
import java.io.IOException;

import java.net.MalformedURLException;
import java.net.URL;

import java.util.Enumeration;
import java.util.TreeSet;

public final class LinearDemo extends JPanel
	implements TreeSelectionListener, TreeExpansionListener, ListSelectionListener {

	private final JTree tree;
	private final JList list;
	private final JFrame htmlFrame;
	private final JEditorPane htmlPane;
	private final URL helpURL;

	private final class FileNode implements Comparable {
		public final String path;
		public final URL url;

		public FileNode(File file, String path) {
			URL url = null;
			try{
				url = file.toURL(); 
			}
			catch(MalformedURLException e){
				e.printStackTrace();
			}
			this.url = url;
			this.path = path;
		}

		public int compareTo(Object o) {
			return path.compareTo(((FileNode)o).path);
		}

		public String toString() {
			return path;
		}
	}

	public LinearDemo(String filename) {
		super(new GridLayout(1,0));

		File file = new File(filename);
		DefaultMutableTreeNode top = createFileNodes(file, "");

		//Create a tree that allows one selection at a time.
		tree = new JTree(top);
		tree.getSelectionModel().setSelectionMode
				(TreeSelectionModel.SINGLE_TREE_SELECTION);

		//Listen for when the selection changes.
		tree.addTreeSelectionListener(this);
		tree.addTreeExpansionListener(this);

		//Create the scroll pane and add the tree to it. 
		JScrollPane treeView = new JScrollPane(tree);

		list = new JList();
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		list.addListSelectionListener(this);
		list.setListData(createList(top));
		JScrollPane listView = new JScrollPane(list);

		//Create the HTML viewing pane.
		htmlFrame = new JFrame();
		htmlFrame.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
		htmlPane = new JEditorPane();
		htmlPane.setEditable(false);
		htmlFrame.setContentPane(new JScrollPane(htmlPane));
		htmlFrame.setSize(new Dimension(1000, 700));

		//Add the scroll panes to a split pane.
		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		splitPane.setLeftComponent(treeView);
		splitPane.setRightComponent(listView);

		splitPane.setPreferredSize(new Dimension(1000, 700));
		splitPane.setDividerLocation(700);

		//Add the split pane to this panel.
		add(splitPane);

		helpURL = null;
	}

	public DefaultMutableTreeNode createFileNodes(File file, String path){
		String filename = file.getName();
		String parentPath = filename + path;
		DefaultMutableTreeNode node =
			new DefaultMutableTreeNode(new FileNode(file, parentPath));
		String childPath = " - " + parentPath;
		if (file.isDirectory()) {
			String[] tabFiles = file.list();
			for (int i = 0; i < tabFiles.length; i++) {
				File childFile = new File(file.getPath() +
										  File.separator + tabFiles[i]);
				node.add(createFileNodes(childFile, childPath));
			}
		}
		return node;
	}

	private Object[] createList(DefaultMutableTreeNode node) {
		TreeSet set = new TreeSet();
		for (Enumeration e = node.depthFirstEnumeration(); e.hasMoreElements();) {
			DefaultMutableTreeNode child = (DefaultMutableTreeNode)e.nextElement();
			if (child.isLeaf()) {
				set.add((FileNode)child.getUserObject());
			}
		}
		return set.toArray();
	}

	public void valueChanged(TreeSelectionEvent e) {
		DefaultMutableTreeNode node =
			(DefaultMutableTreeNode)tree.getLastSelectedPathComponent();
		if (node == null) {
			return;
		}
		if (node.isLeaf()) {
			displayURL(((FileNode)node.getUserObject()).url);
		} else {
			list.setListData(createList(node));
		}
	}

	public void treeExpanded(TreeExpansionEvent e) {
		DefaultMutableTreeNode node =
			(DefaultMutableTreeNode)e.getPath().getLastPathComponent();
		if (node == null) {
			return;
		}
		list.setListData(createList(node));
	}

	public void treeCollapsed(TreeExpansionEvent e) {
	}

	public void valueChanged(ListSelectionEvent e) {
		FileNode node = (FileNode)list.getSelectedValue();
		if (node == null) {
			return;
		}
		displayURL(node.url);
	}

	private void displayURL(URL url) {
		try {
			if (url != null) {
				htmlPane.setPage(url);
				htmlFrame.setVisible(true);
			}
		} catch (IOException e) {
			System.err.println("Attempted to read a bad URL: " + url);
		}
	}

	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(
				UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println("Couldn't use system look and feel.");
		}

		String filename = ".";
		if (args.length > 0)
			filename = args[0];
		
		//Make sure we have nice window decorations.
		JFrame.setDefaultLookAndFeelDecorated(true);

		//Create and set up the window.
		JFrame frame = new JFrame("TreeDemo");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		//Create and set up the content pane.
		LinearDemo newContentPane = new LinearDemo(filename);
		newContentPane.setOpaque(true); //content panes must be opaque
		frame.setContentPane(newContentPane);

		//Display the window.
		frame.pack();
		frame.setVisible(true);
	}
}
